@@ -19,7 +19,7 @@
  * Author: Enrico Zini <enrico@enricozini.com>
  */
 
-struct PrintVars : public BulletinHandler
+struct PrintVars : public BulletinFullHandler
 {
     FILE* out;
     const std::vector<wreport::Varcode>& codes;
@@ -36,11 +36,11 @@ struct PrintVars : public BulletinHandler
     }
 
     /// Dump the contents of a message
-    virtual void handle(const wreport::Bulletin& b)
+    void handle(wreport::Bulletin& b) override
     {
         for (size_t sset = 0; sset < b.subsets.size(); ++sset)
         {
-            fprintf(out, "%s:%zd:", b.fname, sset + 1);
+            fprintf(out, "%s:%zd:", b.fname.c_str(), sset + 1);
             for (size_t i = 0; i < codes.size(); ++i)
             {
                 const Var* var = find_varcode(b.subsets[sset], codes[i]);
